==> xl-react-components-0.18.103.tgz: package/app/xl-react-widget/xl-react-widget-tree-select/tree-utils.js <==
import R from 'ramda';

export const SCROLL_TO_BOTTOM = 'SCROLL_TO_BOTTOM';
export const SCROLL_TO_TOP = 'SCROLL_TO_TOP';

export const initTree = (tree) => {
    const processNode = (node) => {
        if (node.children) {
            return {
                ...node,
                isExpanded: false,
                isParent: node.children.length > 0
            };
        }
        return node;
    };

    const traverse = (node) => {
        const processedNode = processNode(node);
        if (processedNode.children) {
            processedNode.children = R.map(traverse, processedNode.children);
        }
        return processedNode;
    };

    return R.map(traverse, tree);
};

/* eslint-disable complexity */
export const setSelectedNode = (tree, selectedNodeId) => {
    const processNode = ({...node}) => {
        if (node.id === selectedNodeId) {
            if (node.isExpanded) {
                return {
                    ...node,
                    isExpanded: false,
                    isSelected: true
                };
            }
            return {
                ...node,
                isSelected: true
            };
        }
        if (node.children && node.children.some((child) => child.isSelected || child.isExpanded)) {
            return {
                ...node,
                isExpanded: true
            };
        }
        if (node.isExpanded) {
            return {
                ...node,
                isExpanded: false
            };
        }
        return node;
    };

    const traverse = (node) => {
        if (node.children) {
            node.children = R.map(traverse, node.children);
        }
        return processNode(node);
    };

    return R.map(traverse, tree);
};
/* eslint-enable complexity */

export const flatTree = (tree) => {
    const flatten = (root) => {
        const enrichWithDepth = (node) => (child) => ({
            ...child,
            depth: node.depth + 1
        });

        const flattenTree = [];
        let tmp = [];
        let node = {...root, depth: 0};
        while (node) {
            const {children, ...stripedNode} = node;
            flattenTree.push(stripedNode);

            if (stripedNode.isExpanded) {
                tmp = [
                    ...R.map(enrichWithDepth(node), children),
                    ...tmp
                ];
            }
            node = tmp.shift();
        }
        return flattenTree;
    };

    return tree.reduce((acc, curr) => [
        ...acc,
        ...flatten(curr)
    ], []);
};

export const toggleNode = (tree, nodeId, forceState) => {
    const isExpanded = (node) => R.isNil(forceState) ? !node.isExpanded : Boolean(forceState);
    const processNode = (node) => {
        if (node.id === nodeId && !R.isNil(node.isExpanded)) {
            return {
                ...node,
                isExpanded: isExpanded(node)
            };
        }
        return node;
    };

    const traverse = (node) => {
        const processedNode = processNode(node);
        if (node.children) {
            node.children = R.map(traverse, node.children);
        }
        return processedNode;
    };

    return R.map(traverse, tree);
};

export const filterTree = (tree, filter) => {
    const processNode = (node, filteredChildren) => {
        if (filteredChildren.length) {
            return {
                ...node,
                children: filteredChildren,
                isExpanded: true,
                isParent: true
            };
        }
        if (node.title.indexOf(filter) >= 0) {
            return {...node};
        }
        return null;
    };

    const traverse = (node) => {
        let filteredChildren = [];
        if (node.children) {
            filteredChildren = node.children.reduce((acc, curr) => {
                const filtered = traverse(curr);
                if (filtered) {
                    acc.push(filtered);
                }
                return acc;
            }, []);
        }
        return processNode(node, filteredChildren);
    };

    return tree.reduce((acc, curr) => {
        const filtered = traverse(curr);
        if (filtered) {
            acc.push(filtered);
        }
        return acc;
    }, []);
};

const outOfBoundaries = (nodeTop, containerTop, containerHeight) =>
    nodeTop >= containerTop + containerHeight - 4 ||
    nodeTop < containerTop;

const scrollToCurrentItem = (node, container) => {
    const shouldScrollTo = () => container && node &&
        outOfBoundaries(node.offsetTop, container.scrollTop, container.offsetHeight);

    if (shouldScrollTo()) {
        container.scrollTop = node.offsetTop - container.offsetTop;
    }
};

const needScrollToPosition = (container, needScrollTo) => {
    if (needScrollTo === SCROLL_TO_TOP) {
        container.scrollTop = 0;
    }
    if (needScrollTo === SCROLL_TO_BOTTOM) {
        container.scrollTop = container.offsetHeight;
    }
};

export const changeScrollPosition = (node, container, needScrollTo) => {
    const scroll = () => {
        if (R.prop('current', node)) {
            scrollToCurrentItem(node, container);
        } else {
            needScrollToPosition(container, needScrollTo);
        }
    };

    if (container) {
        scroll();
    }
};

==> xl-react-components-0.21.9.tgz: package/app/xl-react-widget/xl-react-widget-tree-select/tree-utils.js <==
import R from 'ramda';

export const SCROLL_TO_BOTTOM = 'SCROLL_TO_BOTTOM';
export const SCROLL_TO_TOP = 'SCROLL_TO_TOP';

export const initTree = (tree, expandAll) => {
    const processNode = (node) => {
        if (node.children) {
            return {
                ...node,
                isExpanded: Boolean(expandAll),
                isParent: node.children.length > 0
            };
        }
        return node;
    };

    const traverse = (node) => {
        const processedNode = processNode(node);
        if (processedNode.children) {
            processedNode.children = R.map(traverse, processedNode.children);
        }
        return processedNode;
    };

    return R.map(traverse, tree);
};

/* eslint-disable complexity */
export const setSelectedNode = (tree, selectedNodeId, expandAll) => {
    const processNode = ({...node}) => {
        if (node.id === selectedNodeId) {
            return {
                ...node,
                isExpanded: node.children ? Boolean(expandAll) : false,
                isSelected: true
            };
        }
        if (node.children && node.children.some((child) => child.isSelected || child.isExpanded)) {
            return {
                ...node,
                isExpanded: true
            };
        }

        return {
            ...node,
            isExpanded: node.children ? Boolean(expandAll) : false
        };
    };

    const traverse = (node) => {
        if (node.children) {
            node.children = R.map(traverse, node.children);
        }
        return processNode(node);
    };

    return R.map(traverse, tree);
};
/* eslint-enable complexity */

export const flatTree = (tree) => {
    const flatten = (root) => {
        const enrichWithDepth = (node) => (child) => ({
            ...child,
            depth: node.depth + 1
        });

        const flattenTree = [];
        let tmp = [];
        let node = {...root, depth: 0};
        while (node) {
            const {children, ...stripedNode} = node;
            flattenTree.push(stripedNode);

            if (stripedNode.isExpanded) {
                tmp = [
                    ...R.map(enrichWithDepth(node), children),
                    ...tmp
                ];
            }
            node = tmp.shift();
        }
        return flattenTree;
    };

    return tree.reduce((acc, curr) => [
        ...acc,
        ...flatten(curr)
    ], []);
};

export const toggleNode = (tree, nodeId, forceState) => {
    const isExpanded = (node) => R.isNil(forceState) ? !node.isExpanded : Boolean(forceState);
    const processNode = (node) => {
        if (node.id === nodeId && !R.isNil(node.isExpanded)) {
            return {
                ...node,
                isExpanded: isExpanded(node)
            };
        }
        return node;
    };

    const traverse = (node) => {
        const processedNode = processNode(node);
        if (node.children) {
            node.children = R.map(traverse, node.children);
        }
        return processedNode;
    };

    return R.map(traverse, tree);
};

export const filterTree = (tree, filter) => {
    const processNode = (node, filteredChildren) => {
        if (filteredChildren.length) {
            return {
                ...node,
                children: filteredChildren,
                isExpanded: true,
                isParent: true
            };
        }
        if (node.title.indexOf(filter) >= 0) {
            return {...node};
        }
        return null;
    };

    const traverse = (node) => {
        let filteredChildren = [];
        if (node.children) {
            filteredChildren = node.children.reduce((acc, curr) => {
                const filtered = traverse(curr);
                if (filtered) {
                    acc.push(filtered);
                }
                return acc;
            }, []);
        }
        return processNode(node, filteredChildren);
    };

    return tree.reduce((acc, curr) => {
        const filtered = traverse(curr);
        if (filtered) {
            acc.push(filtered);
        }
        return acc;
    }, []);
};

const outOfBoundaries = (nodeTop, containerTop, containerHeight) =>
    nodeTop >= containerTop + containerHeight - 4 ||
    nodeTop < containerTop;

const scrollToCurrentItem = (node, container) => {
    const shouldScrollTo = () => container && node &&
        outOfBoundaries(node.offsetTop, container.scrollTop, container.offsetHeight);

    if (shouldScrollTo()) {
        container.scrollTop = node.offsetTop - container.offsetTop;
    }
};

const needScrollToPosition = (container, needScrollTo) => {
    if (needScrollTo === SCROLL_TO_TOP) {
        container.scrollTop = 0;
    }
    if (needScrollTo === SCROLL_TO_BOTTOM) {
        container.scrollTop = container.offsetHeight;
    }
};

export const changeScrollPosition = (node, container, needScrollTo) => {
    const scroll = () => {
        if (R.prop('current', node)) {
            scrollToCurrentItem(node, container);
        } else {
            needScrollToPosition(container, needScrollTo);
        }
    };

    if (container) {
        scroll();
    }
};
